#!/usr/bin/env node
"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const readline_1 = __importDefault(require("readline"));
const ora_1 = __importDefault(require("ora"));
const opn_1 = __importDefault(require("opn"));
const axios_1 = __importDefault(require("axios"));
const cheerio_1 = __importDefault(require("cheerio"));
const chalk_1 = __importDefault(require("chalk"));
const url_1 = __importDefault(require("url"));
const getArticle = ({ article, title }) => __awaiter(this, void 0, void 0, function* () {
    const res = yield axios_1.default.get(article);
    return new Promise((resolve, reject) => {
        if (res.status === 200) {
            const $ = cheerio_1.default.load(res.data);
            const content = [chalk_1.default.cyan(title) + '\n'];
            const heads = ['h1', 'h2', 'h3', 'h4', 'h5', 'h6'];
            // TODO: Better text formatting
            $('.block.article-body .block-content')
                .children()
                .each((i, elem) => {
                if (heads.includes(elem.tagName)) {
                    content.push('\n' + chalk_1.default.cyan($(elem).text()));
                }
                else if (elem.tagName === 'p') {
                    content.push($(elem).text());
                }
            });
            resolve('\n' + content.join('\n'));
        }
        else {
            reject(`Unable to get article content: ${res.status} ${res.statusText}`);
        }
    });
});
const inlineMode = (argv) => argv.slice(2).some(arg => ['-I', '-i', '--inline', '-inline'].includes(arg));
const showList = (list) => {
    const output = list
        .map(({ title }, index) => `${index + 1}) ${title}`)
        .join('\n');
    process.stdout.write(`\n${output}\n`);
};
const promptQuestion = (list) => new Promise((resolve, reject) => {
    const rl = readline_1.default.createInterface({
        input: process.stdin,
        output: process.stdout,
    });
    const question = () => {
        rl.question('\nSelect an article (q to exit): ', answer => {
            if (answer === 'q') {
                rl.close();
                process.exit();
            }
            else {
                const index = parseInt(answer, 10);
                if (index > 0 && index <= list.length) {
                    rl.close();
                    resolve(list[index - 1]);
                }
                else {
                    question();
                }
            }
        });
    };
    question();
});
const addItem = (items, title, link) => {
    if (title && link) {
        const article = url_1.default.resolve('https://www.nu.nl', link);
        items.push({ title, article });
    }
};
const getHeadlines = (link) => __awaiter(this, void 0, void 0, function* () {
    const res = yield axios_1.default.get(link);
    return new Promise((resolve, reject) => {
        if (res.status === 200) {
            const $ = cheerio_1.default.load(res.data);
            const items = [];
            const headline = $('.block.headline.section-nu > a');
            addItem(items, $('.block.headline.section-nu > a h1.title').text(), $('.block.headline.section-nu > a').attr('href'));
            $('.block.articlelist .section-nu.source-normal li').each((i, elem) => {
                // .subtitle contains Video, Podcast, In beeld etc.
                if ($(elem)
                    .find('.subtitle')
                    .text().length) {
                    return;
                }
                addItem(items, $(elem)
                    .find('.title')
                    .text(), $(elem)
                    .find('>a')
                    .attr('href'));
            });
            resolve(items);
        }
        else {
            reject(`Unable to get article content: ${res.status} ${res.statusText}`);
        }
    });
});
const main = () => __awaiter(this, void 0, void 0, function* () {
    const spinner = ora_1.default({
        color: 'white',
        text: 'Loading news',
    }).start();
    const items = yield getHeadlines('https://www.nu.nl');
    spinner.stop();
    showList(items);
    const selected = yield promptQuestion(items);
    // Open in browser by default
    if (!inlineMode(process.argv)) {
        opn_1.default(selected.article);
        process.exit();
    }
    spinner.start('Loading article');
    const articleText = yield getArticle(selected);
    spinner.stop();
    process.stdout.write(`${articleText}\n`);
    process.exit();
});
main().catch(console.error);
